#include <esp_now.h>
#include <esp_wifi.h>

#include <Arduino.h>
#include <TM1637Display.h>

#include <WiFi.h>
#include <time.h>

#define PIN_BUZZER 17
#define CHN       0  

// Module connection pins (Digital Pins)
#define CLK 18
#define DIO 19

uint8_t broadcastAddress[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}; //broadcast

int b = 0;
int c = 0;

const char* ssid       = ":)";
const char* password   = "0967810678";

const char* ntpServer = "th.pool.ntp.org";
const long  gmtOffset_sec = 7 * 3600;
const int   daylightOffset_sec = 0;

typedef struct test_struct {
  int alarm_sender;
  int alarm_receive;
} test_struct;

TM1637Display display(CLK, DIO);

void OnDataRecv(const uint8_t * mac, const uint8_t *data, int len) {
   
  test_struct* test =(test_struct*) data;
   
  Serial.println(test->alarm_receive);

  if (test->alarm_receive == 2)
  {
    b = 0;
    c = 1;
  }
}

void buzzer() {
  ledcWriteTone(CHN,1500);
  delay(500);
  uint8_t octave = 1;
  ledcWriteNote(0,NOTE_C,octave); 
  delay(500); 
}

void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}

void sender_alarm()
{ 
  test_struct test;
  test.alarm_sender = 1;

  if (test.alarm_receive == 2) {
    test.alarm_sender = 0;
  }
 
  esp_err_t result = esp_now_send(
    broadcastAddress, 
    (uint8_t *) &test,
    sizeof(test_struct));
   
  if (result == ESP_OK) {
    Serial.println("Sent with success");
  }
  else {
    Serial.println("Error sending the data");
  }
}

void setup()
{
  Serial.begin(115200);

  pinMode(PIN_BUZZER, OUTPUT);
  ledcSetup(CHN, 0, 10);
  ledcAttachPin(PIN_BUZZER, CHN);
  
  //connect to WiFi
  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  Serial.println(" CONNECTED");

  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
  
  //ESP NOW
  WiFi.mode(WIFI_STA);
  
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
   
  // register peer
  esp_now_peer_info_t peerInfo;
    
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 6;  
  peerInfo.encrypt = false;
          
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
}

void loop ()
{ 
//  delay(500);
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }

  uint8_t data[] = { 0xff, 0xff, 0xff, 0xff };
  display.setBrightness(0x0f);

  data[0] = display.encodeDigit(timeinfo.tm_hour / 10);
  data[1] = display.encodeDigit(timeinfo.tm_hour % 10) | (timeinfo.tm_sec % 2 ? 0x80 : 0x00);
  data[2] = display.encodeDigit(timeinfo.tm_min / 10);
  data[3] = display.encodeDigit(timeinfo.tm_min % 10);
  display.setSegments(data);
  Serial.println(&timeinfo , "%H:%M"); 
  esp_now_register_recv_cb(OnDataRecv);

  if(timeinfo.tm_hour == 21 &&
     timeinfo.tm_min == 35)
     {
      if (c == 1) {
        b == 0;
      }
      else {
        b = 1;
      }
     }

     if (b == 1)
     {
      Serial.println("yes");
      sender_alarm();
      buzzer();
     }

//  buzzer();
//  sender();
}
